'=============================================================================
' Copyright  Micromega Corporation 2002-2004. All rights reserved
'
' File: graph-spi.bsp
'
' This program demonstrates the use of the uM-FPU V2 floating point coprocessor
' with the PicBasic Pro compiler using a SPI interface. It draws a simple graph
' of the sine and cosine functions using the DEBUG command.
'
' @author Cam Thompson, Micromega Corporation
' @version V2.0 - January 17, 2005
' changes:
'   V2.0 - January 17, 2005
'     - modified for PicBasic Pro compiler
'   V1.0 - January 8, 2005
'     - original version
'==============================================================================

'-------------------- PIC oscillator speed ------------------------------------
define      OSC 20                      'specify the speed of the oscillator
OSC_SPEED   con 20                      'define PicBasic constant

'-------------------- debug definitions ---------------------------------------
define      DEBUG_REG   PORTC
define      DEBUG_BIT   6
define      DEBUG_BAUD  9600
define      DEBUG_MODE  0

'-------------------- uM-FPU pin definitions ----------------------------------
FpuClk      var     PORTC.3
FpuIn       var     PORTC.4
FpuOut      var     PORTC.4

include     "umfpuV2-spi.bas"           'include the uM-FPU support routines

'-------------------- uM-FPU register definitions -----------------------------
Angle           con     1               'angle          (uM-FPU register 1)
Point           con     2               'point on graph (uM-FPU register 2)

'-------------------- constants ----------------------------------------------

Space           con     32              'ASCII character values
Asterisk        con     42
VerticalLine    con     124

'-------------------- variables ----------------------------------------------

cnt             var     byte            'counter
tmp             var     byte            'temporary value
x               var     byte            'x value for graph
p               var     byte            'point value for graph

'=============================================================================
'-------------------- initialization -----------------------------------------
'=============================================================================

Reset:
  DEBUG 13, 10, 13, 10, "Graph-SPI", 13, 10

  GOSUB Fpu_Reset                       'reset the FPU hardware
  IF fpu_status <> SyncChar THEN
    DEBUG "uM-FPU not detected."
    END
  ELSE
    GOSUB Print_Version                 'display the uM-FPU version number
    DEBUG 13, 10
  ENDIF

'=============================================================================
'-------------------- main routine -------------------------------------------
'=============================================================================

Main:
  'graph sine from 0 TO 360 degrees (2*PI radians)
  '-----------------------------------------------
  DEBUG 13, 10, "Graph of sine"
  DEBUG 13, 10, "-1        0        1"
  DEBUG 13, 10, "--------------------", 13, 10
  FOR cnt = 0 TO 20
    'Angle = sin(pi * 2 * cnt / 20)
    '------------------------------
    GOSUB Fpu_Wait
    SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA+Angle, XOP, LOADPI, FSET]
    SHIFTOUT FpuOut, FpuClk, MSBFIRST, [LOADBYTE, cnt*2, FMUL, LOADBYTE, 20, FDIV, FSIN]
    GOSUB Graph                         'graph the result
  NEXT

  'graph cosine from 0 TO 360 degrees (2*PI radians)
  '-------------------------------------------------
  DEBUG 13, 10, "Graph of cosine"
  DEBUG 13, 10, "-1        0        1"
  DEBUG 13, 10, "--------------------", 13, 10
  FOR cnt = 0 TO 20
    'Angle = cos(pi * 2 * cnt / 20)
    '------------------------------
    GOSUB Fpu_Wait
    SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA+Angle, XOP, LOADPI, FSET]
    SHIFTOUT FpuOut, FpuClk, MSBFIRST, [LOADBYTE, cnt*2, FMUL, LOADBYTE, 20, FDIV, FCOS]
    GOSUB Graph                         'graph the result
  NEXT

  'End of program
  '--------------   
  debug 13, 10, "Done.", 13, 10         'end of program
  end

'-------------------- Graph --------------------------------------------------

Graph:
  'Point = Fix(Round(Angle * 10))
  '------------------------------
  GOSUB Fpu_Wait
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA+Point, FSET+Angle, LOADBYTE, 10, FMUL, ROUND, FIX]

  'Read the 32-bit integer (only use lower 8 bits)
  '-----------------------------------------------
  GOSUB Fpu_Wait
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA+0, XOP, READBYTE]
  GOSUB Fpu_ReadDelay
  SHIFTIN FpuIn, FpuClk, MSBPRE, [p]

  'add offset to midpoint of graph
  '-------------------------------
  p = p + 10

  'plot the point on graph
  '-----------------------
 
  FOR x = 0 TO 20                       'plot the point on graph
    tmp = Space
    IF (x = 10) THEN tmp = VerticalLine
    IF (x = p) THEN tmp = Asterisk
    DEBUG tmp
  NEXT

  'display the original angle in 8.5 format
  '----------------------------------------
  DEBUG Space
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA+Angle]
  format = 85     '(8.5 format i.e. width=8, decimals=5)
  GOSUB Print_FloatFormat
  DEBUG 13, 10
  RETURN
