'==============================================================================
' Copyright  Micromega Corporation 2002-2004. All rights reserved
'
' File: sample-spi.bsp
'
' Demonstration routine for uM-FPU V2 using a SPI interface.
' The demo program takes an integer value representing the diameter
' of a circle in centimeters, converts the value to inches, then calculates
' the circumference and area in inches and square inches.
'
' @author Cam Thompson, Micromega Corporation
' @version V2.0 - V2.0 - January 8, 2005
' changes:
'   V2.0 - January 17, 2005
'     - modified for PicBasic Pro compiler
'   V1.0 - April 29, 2004
'     - initial version
'==============================================================================

'-------------------- PIC oscillator speed ------------------------------------
define      OSC 20                      'specify the speed of the oscillator
OSC_SPEED   con 20                      'define PicBasic constant

'-------------------- debug definitions ---------------------------------------
define      DEBUG_REG   PORTC
define      DEBUG_BIT   6
define      DEBUG_BAUD  9600
define      DEBUG_MODE  0

'-------------------- uM-FPU pin definitions ----------------------------------
FpuClk      var     PORTC.3
FpuIn       var     PORTC.4
FpuOut      var     PORTC.4

include     "umfpuV2-spi.bas"           'include the uM-FPU support routines

'-------------------- uM-FPU register definitions -----------------------------

DiameterIn      con     1               'dimeter in inches
Circumference   con     2               'circumference
Area            con     3               'area
Pi              con     4               'constant pi
F2_0            con     5               'constant 2.0
F2_54           con     6               'constant 2.54

'-------------------- variables -----------------------------------------------

diameterCm      var     byte            'diameter in centimeters

'------------------------------------------------------------------------------
' The following example takes an integer value representing the diameter
' of a circle in centimeters, converts the value to inches and calculates
' the circumference and area in inches and square inches.
'------------------------------------------------------------------------------

'=============================================================================
'-------------------- initialization -----------------------------------------
'=============================================================================

Reset:
  DEBUG 13, 10, 13, 10, "Conversion Example"
  DEBUG 13, 10, "------------------", 13, 10

  '------------------------------------
  ' Reset the uM-FPU and print version.
  '------------------------------------

  GOSUB Fpu_Reset                       'reset the FPU hardware
  IF fpu_status <> SyncChar THEN
    DEBUG "uM-FPU not detected."
    END
  ELSE
    GOSUB Print_Version                 'display the uM-FPU version number
    DEBUG 13, 10
  ENDIF

  '-----------------------------------------------------
  ' Load constants for later use.
  ' Three different ways of loading constants are shown.
  '-----------------------------------------------------
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [Pi, XOP, LOADPI, FSET]
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [F2_0, LOADBYTE, 2, FSET]
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [F2_54, ATOF, "2.54", 0, FSET]

'=============================================================================
'-------------------- main routine -------------------------------------------
'=============================================================================

Main:
  '----------------------------------------------------------------------------
  ' Get diameter in centimeters.  The value would typically come from a sensor
  ' reading, but in this example an assumed value of 25 is used.
  '----------------------------------------------------------------------------
  diameterCm = 25
  DEBUG 13, 10, "Diameter (cm):       ", DEC diameterCm

  '------------------------------
  ' diameterIn = diameterCm / 2.54
  '------------------------------
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [DiameterIn, LOADBYTE, diameterCm, FSET, FDIV+F2_54]
  DEBUG 13, 10, "Diameter (in.):      "
  GOSUB Print_Float

  '------------------------------
  ' circumference = diameter * pi
  '------------------------------
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [Circumference, FSET+DiameterIn, FMUL+Pi]
  DEBUG 13, 10, "Circumference (in.): "
  GOSUB Print_Float

  '----------------------------
  'area = (diameter / 2)^2 * pi
  '----------------------------
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [Area, FSET+DiameterIn, FDIV+F2_0, FMUL+Area, FMUL+Pi]
  DEBUG 13, 10, "Area (sq.in.):       "
  GOSUB Print_Float

  DEBUG 13, 10, "Done.", 13, 10         'end of program
  END
