'==============================================================================
' Copyright  Micromega Corporation 2004-2005. All rights reserved
'
' File: template-spi.bas
'
' This file is intended to be template for developing applications using the
' uM-FPU V2 floating point coprocessor with the meLabs PIC Basic Pro compiler.
' The include file umfpuV2-spi.bas contains all of the support code for using
' the uM-FPU with a SPI interface. The template program resets the uM-FPU,
' confirms communications, and prints out the version number.  It can be used
' as a quick initial test that the uM-FPU is running, and the file can also be
' used as a template for developing your own applications.
'
' @author Cam Thompson, Micromega Corporation
' @version V2.0 - January 17, 2005
' changes:
'   V2.0 - January 17, 2005
'     - modified for uM-FPU V2.0
'   V1.0 - January 14, 2005
'     - initial version
'==============================================================================

'-------------------- PIC oscillator speed ------------------------------------
' The oscillator speed must be defined to ensure proper timing constants
' for the uM-FPU routines.

define      OSC 20                      'specify the speed of the oscillator
OSC_SPEED   con 20                      'define PicBasic constant

'-------------------- debug definitions ---------------------------------------
' The serial pin and baud rate for the DEBUG command are specified with the
' following defines. These should be changed to suit your application.

define      DEBUG_REG   PORTC
define      DEBUG_BIT   6
define      DEBUG_BAUD  9600
define      DEBUG_MODE  0

'-------------------- uM-FPU pin definitions ----------------------------------
' The following assignments define the pins used to connect to the uM-FPU V2.0
' chip. These should be changed to suit your application.

FpuClk      var     PORTC.3
FpuIn       var     PORTC.4
FpuOut      var     PORTC.4

'-------------------- uM-FPU register definitions -----------------------------
' The uM-FPU registers 1 to 15 can be assigned to suit your application.
' The following example defines uM-FPU register 1 as the Result register.

Result      con     1                   'uM-FPU register 1

'-------------------- uM-FPU include ------------------------------------------
' The umfpuV2-spi.bas file contains the definitions and support code for using
' the uM-FPU V2 floating point coprocessor with a SPI interface.

include "umfpuV2-spi.bas"

'-------------------- initialization ------------------------------------------

reset:
  debug 13, 10, 13, 10, "Template-SPI", 13, 10

  GOSUB Fpu_Reset                       'reset the FPU hardware
  IF fpu_status <> SyncChar THEN
    DEBUG "uM-FPU not detected."
    END
  ELSE
    GOSUB Print_Version                 'display the uM-FPU version number
    DEBUG 13, 10
  ENDIF

  '-----------------------------------------------
  ' Floating point constants would be loaded here.
  ' The following code is a simple example.
  '-----------------------------------------------

  'Set Result register to the value pi.
  '------------------------------------
  shiftout FpuOut, FpuClk, MSBFIRST, [Result, XOP, LOADPI, FSET]  

'-------------------- main routine --------------------------------------------

main:
  '-----------------------------------------
  'Your main routine would be inserted here.
  'The following code is a simple example.
  '-----------------------------------------

  'Print the value of Result (previously set to pi)
  '------------------------------------------------
  shiftout FpuOut, FpuClk, MSBFIRST, [Result] 
  debug 13, 10, "Result = "                         
  gosub Print_Float

  'Calculate Result = Result / 2.0
  '-------------------------------
  shiftout FpuOut, FpuClk, MSBFIRST, [Result, LOADBYTE, 2, FDIV]                            

  'Print the value of Result (pi/2)
  '--------------------------------
  debug 13, 10, "Result = "
  gosub Print_Float

  'End of program
  '--------------   
  debug 13, 10, "Done.", 13, 10         'end of program
  end

